@@ -7,6 +7,7 @@ import sys
 from typing import Any
 
 from rich.console import Console
+from rich.markup import escape
 from textual.app import App, ComposeResult
 from textual.binding import Binding
 from textual.containers import Horizontal, Vertical
@@ -42,10 +43,12 @@ class DocumentApp(App[None]):
         self.current_approval_dialog: ApprovalDialog | None = None
         self.current_approval_backdrop: ApprovalBackdrop | None = None
         self.current_error_panel: ErrorPanel | None = None
+        self.header_widget: DocumentHeader | None = None
 
     def compose(self) -> ComposeResult:
         with Vertical(id="top-bar"):
-            yield DocumentHeader(id="header")
+            self.header_widget = DocumentHeader(id="header")
+            yield self.header_widget
             with Horizontal(id="toolbar"):
                 yield Button("Send", id="submit-btn")
                 yield Button("Status", id="status-btn")
@@ -102,11 +105,9 @@ class DocumentApp(App[None]):
         elif button_id == "approval-allow":
             self.handle_approval_response("allow")
         elif button_id == "approval-yes":
-            self.handle_approval_response("y")
+            self.handle_approval_response("yes")
         elif button_id == "approval-no":
-            self.handle_approval_response("n")
-        elif button_id == "approval-stop":
-            self.handle_approval_response("stop")
+            self.handle_approval_response("no")
         elif button_id == "error-ok":
             if self.current_error_panel:
                 self.current_error_panel.remove()
@@ -186,6 +187,11 @@ class DocumentApp(App[None]):
         except Exception:
             status_bar.update_status("unknown", 0, 0)
 
+    def refresh_header(self) -> None:
+        """Refresh the document header with updated current working directory."""
+        if self.header_widget:
+            self.header_widget.refresh_cwd()
+
     def request_approval(
         self, tool_name: str, tool_input: dict[str, Any], diff_content: str | None = None
     ) -> bool:
@@ -199,9 +205,9 @@ class DocumentApp(App[None]):
         input_text = "\n".join(input_lines)
 
         def write_prompt() -> None:
-            conv_log.write(f"\n[bold cyan]→ {tool_name}[/bold cyan]")
+            conv_log.write(f"\n[bold cyan]→ {escape(tool_name)}[/bold cyan]")
             if input_text:
-                conv_log.write(f"[cyan]{input_text}[/cyan]")
+                conv_log.write(f"[cyan]{escape(input_text)}[/cyan]")
 
             # Mention that diff preview is available in the approval dialog
             # to avoid duplicate display of the same information
@@ -255,7 +261,7 @@ class DocumentApp(App[None]):
 
         self.call_from_thread(show_input)
 
-        if response == "stop":
+        if response == "no":
             raise InterruptedExceptionError()
         elif response == "allow":
             # Check if this is an MCP tool
@@ -276,7 +282,7 @@ class DocumentApp(App[None]):
                     else:
                         conv_log.write("[yellow]⚠ MCP manager not available[/yellow]")
                 except Exception as e:
-                    conv_log.write(f"[yellow]⚠ Error trusting server: {e}[/yellow]")
+                    conv_log.write(f"[yellow]⚠ Error trusting server: {escape(str(e))}[/yellow]")
                 return True
             else:
                 # Auto-approve this tool type for non-MCP tools
@@ -303,13 +309,15 @@ class DocumentApp(App[None]):
                     self.agent.permission_manager.update_permission(
                         action_type, PermissionLevel.AUTO_APPROVE
                     )
-                    conv_log.write(f"[green]Auto-approving {tool_name} for this session[/green]")
+                    conv_log.write(
+                        f"[green]Auto-approving {escape(tool_name)} for this session[/green]"
+                    )
                     return True
                 else:
                     # Fallback to regular approval
                     return True
 
-        return response == "y"
+        return response == "yes"
 
     def action_submit(self) -> None:
         conv_log = self.query_one("#conversation-log", RichLog)
@@ -322,16 +330,38 @@ class DocumentApp(App[None]):
         # Check if waiting for approval
         if self.waiting_for_approval:
             response = user_input.lower()
-            if response in ["y", "n", "stop", "allow", "a"]:
-                # Convert shorthand responses
-                if response == "a":
-                    response = "allow"
-                self.approval_queue.put(response)
+
+            # Empty input - just reprompt (do nothing, clear input)
+            if response == "":
                 user_input_widget.value = ""
                 return
 
-        # Show user input
-        conv_log.write(f"[bold][You] ➜[/bold] {user_input}")
+            valid_responses = {"yes", "y", "no", "n", "allow", "a"}
+
+            if response in valid_responses:
+                # Convert shorthand responses
+                if response == "a":
+                    response = "allow"
+                elif response == "y":
+                    response = "yes"
+                elif response == "n":
+                    response = "no"
+                self.approval_queue.put(response)
+                user_input_widget.value = ""
+                return
+            else:
+                # Show error message for invalid response
+                def show_error() -> None:
+                    conv_log.write(
+                        "[yellow]Invalid response. Please enter (y)es, (n)o, or (a)llow.[/yellow]"
+                    )
+
+                self.call_from_thread(show_error)
+                user_input_widget.value = ""
+                return
+
+        # Show user input (escape to prevent markup interpretation)
+        conv_log.write(f"[bold][You] ➜[/bold] {escape(user_input)}")
         conv_log.write("")
         user_input_widget.value = ""
 
@@ -362,7 +392,7 @@ class DocumentApp(App[None]):
                     f"{stats['messages_after']} (summarized {stats['messages_summarized']})"
                 )
             else:
-                conv_log.write(f"[yellow]⚠ Cannot Compact: {message}[/yellow]")
+                conv_log.write(f"[yellow]⚠ Cannot Compact: {escape(message)}[/yellow]")
             return
         elif user_input.lower().startswith("/model"):
             self.handle_model_command(user_input)
@@ -419,9 +449,11 @@ class DocumentApp(App[None]):
                 self.agent.permission_manager.update_permission(
                     action_type, PermissionLevel.REQUIRE_APPROVAL
                 )
-                conv_log.write(f"[green]Revoked auto-approval for {action_to_revoke}[/green]")
+                conv_log.write(
+                    f"[green]Revoked auto-approval for {escape(action_to_revoke)}[/green]"
+                )
             else:
-                conv_log.write(f"[red]Unknown action type: {action_to_revoke}[/red]")
+                conv_log.write(f"[red]Unknown action type: {escape(action_to_revoke)}[/red]")
         elif subcommand == "clear":
             # Clear all auto-approvals
             from ..permissions import PermissionLevel
@@ -476,7 +508,7 @@ class DocumentApp(App[None]):
         elif subcommand == "revoke":
             self._handle_mcp_revoke(mcp_manager, conv_log, subcommand_args)
         else:
-            conv_log.write(f"[red]Unknown MCP command: {subcommand}[/red]")
+            conv_log.write(f"[red]Unknown MCP command: {escape(subcommand)}[/red]")
             conv_log.write(
                 "[dim]Available commands: list, tools, status, refresh, allow, revoke[/dim]"
             )
@@ -496,7 +528,7 @@ class DocumentApp(App[None]):
                 "[green]connected[/green]" if server["connected"] else "[red]disconnected[/red]"
             )
             conv_log.write(
-                f"• [cyan]{server['server_id']:20}[/cyan] - {status} "
+                f"• [cyan]{escape(server['server_id']):20}[/cyan] - {status} "
                 f"({server['tools_count']} tools)"
             )
         conv_log.write("")
@@ -507,7 +539,7 @@ class DocumentApp(App[None]):
             # List tools for specific server
             tools = mcp_manager.list_tools(server_arg)
             if not tools:
-                conv_log.write(f"[yellow]No tools found for server '{server_arg}'[/yellow]")
+                conv_log.write(f"[yellow]No tools found for server '{escape(server_arg)}'[/yellow]")
                 return
         else:
             # List tools for all servers
@@ -521,8 +553,9 @@ class DocumentApp(App[None]):
         for tool in tools:
             if tool["server_id"] != current_server:
                 current_server = tool["server_id"]
-                conv_log.write(f"\n[bold]Server: {current_server}[/bold]")
-            conv_log.write(f"  • [cyan]{tool['name']}[/cyan] - {tool['description']}")
+                conv_log.write(f"\n[bold]Server: {escape(current_server)}[/bold]")
+            tool_desc = f"  • [cyan]{escape(tool['name'])}[/cyan] - {escape(tool['description'])}"
+            conv_log.write(tool_desc)
         conv_log.write("")
 
     def _handle_mcp_status(self, mcp_manager: Any, conv_log: RichLog) -> None:
@@ -545,7 +578,7 @@ class DocumentApp(App[None]):
             status_color = "green" if connected else "red"
             status_text = "connected" if connected else "disconnected"
 
-            conv_log.write(f"\n[bold]{status_symbol} {server_id}[/bold]")
+            conv_log.write(f"\n[bold]{status_symbol} {escape(server_id)}[/bold]")
             conv_log.write(f"  Connection: [{status_color}]{status_text}[/{status_color}]")
             conv_log.write(
                 f"  Trusted: [{'green' if trusted else 'yellow'}]"
@@ -558,7 +591,7 @@ class DocumentApp(App[None]):
                 tools = mcp_manager.list_tools(server_id)
                 conv_log.write("  Available tools:")
                 for tool in tools[:5]:  # Show first 5 tools
-                    conv_log.write(f"    • {tool['name']}")
+                    conv_log.write(f"    • {escape(tool['name'])}")
                 if tools_count > 5:
                     conv_log.write(f"    ... and {tools_count - 5} more")
 
@@ -575,7 +608,7 @@ class DocumentApp(App[None]):
             # Refresh and show updated status
             self._handle_mcp_list(mcp_manager, conv_log)
         except Exception as e:
-            conv_log.write(f"[red]✗ Error refreshing MCP servers: {e}[/red]")
+            conv_log.write(f"[red]✗ Error refreshing MCP servers: {escape(str(e))}[/red]")
 
     def _handle_mcp_allow(self, mcp_manager: Any, conv_log: RichLog, server_arg: str) -> None:
         """Handle /mcp allow command."""
@@ -586,7 +619,7 @@ class DocumentApp(App[None]):
         server_id = server_arg.strip()
         mcp_manager.set_trusted(server_id, True)
         conv_log.write(
-            f"[green]✓ Marked MCP server '{server_id}' as trusted for this session[/green]"
+            f"[green]✓ Marked MCP server '{escape(server_id)}' as trusted for this session[/green]"
         )
 
     def _handle_mcp_revoke(self, mcp_manager: Any, conv_log: RichLog, server_arg: str) -> None:
@@ -597,7 +630,7 @@ class DocumentApp(App[None]):
 
         server_id = server_arg.strip()
         mcp_manager.set_trusted(server_id, False)
-        conv_log.write(f"[green]✓ Revoked trust for MCP server '{server_id}'[/green]")
+        conv_log.write(f"[green]✓ Revoked trust for MCP server '{escape(server_id)}'[/green]")
 
     async def run_agent_async(self, user_input: str) -> None:
         """Run agent in thread, write output directly to log."""
@@ -631,11 +664,11 @@ class DocumentApp(App[None]):
                                 # Hide thinking indicator when output arrives
                                 self.app.call_from_thread(self.app.hide_thinking)
 
-                                # Write the line
+                                # Write the line (escape to prevent markup interpretation)
                                 self.app.call_from_thread(
                                     lambda t=clean_text: self.app.query_one(
                                         "#conversation-log", RichLog
-                                    ).write(t)
+                                    ).write(escape(t))
                                 )
 
                                 # Show thinking indicator again after tool results
@@ -695,7 +728,7 @@ class DocumentApp(App[None]):
         except Exception as err:
             error_msg = str(err)
             # Write directly since we're in async context, not a separate thread
-            conv_log.write(f"\n[red]Error: {error_msg}[/red]")
+            conv_log.write(f"\n[red]Error: {escape(error_msg)}[/red]")
         finally:
             log_writer.flush()  # Flush any remaining buffer
             sys.stdout = old_stdout
@@ -758,10 +791,10 @@ class DocumentApp(App[None]):
         conv_log.write("[bold]✅ Approval System[/bold]")
         conv_log.write("• When a tool requires approval, you'll see a yellow warning")
         conv_log.write(
-            "• Type [bold]y[/bold] (yes), [bold]n[/bold] (no), or [bold]stop[/bold] to interrupt"
+            "• Type [bold](y)es[/bold] to approve, [bold](n)o[/bold] to reject and stop execution"
         )
         conv_log.write(
-            "• Type [bold]a[/bold] or [bold]allow[/bold] to approve and auto-approve future calls"
+            "• Type [bold](a)llow[/bold] to approve and auto-approve all future calls of this type"
         )
         conv_log.write("• File operations (write, delete) and commands need approval")
         conv_log.write("• Read operations are auto-approved")
@@ -853,11 +886,14 @@ class DocumentApp(App[None]):
 
         conv_log.write("\n📎 [bold]Available Model Presets[/bold]\n")
 
-        for name, desc in models:
+        for name, desc, is_default in models:
+            default_indicator = " [green](default)[/green]" if is_default else ""
             if name == current_model:
-                conv_log.write(f"• [green]★ {name:20}[/green] - {desc} [dim](current)[/dim]")
+                conv_log.write(
+                    f"• [green]★ {name:20}[/green] - {desc}{default_indicator} [dim](current)[/dim]"
+                )
             else:
-                conv_log.write(f"• [cyan]{name:20}[/cyan] - {desc}")
+                conv_log.write(f"• [cyan]{name:20}[/cyan] - {desc}{default_indicator}")
 
         conv_log.write("")
         conv_log.write("[bold]Current Configuration:[/bold]")
@@ -880,15 +916,23 @@ class DocumentApp(App[None]):
             self.show_models()
         else:
             model_name = parts[1].strip()
-            config = get_model_config(model_name)
-            if config:
-                api_key = os.getenv(config.api_key_env) or "not-set"
+            model_config, provider_config = get_model_config(model_name)
+            if model_config and provider_config:
+                api_key = os.getenv(provider_config.api_key_env) or "not-set"
                 success, message = self.agent.switch_model(
-                    model=config.model_id, base_url=config.base_url, api_key=api_key
+                    model=model_config.model_id,
+                    base_url=provider_config.base_url,
+                    api_key=api_key,
                 )
             else:
                 success, message = self.agent.switch_model(model=model_name)
-            conv_log.write(f"[green]✓ {message}[/green]" if success else f"[red]✗ {message}[/red]")
+
+            status_msg = (
+                f"[green]✓ {escape(message)}[/green]"
+                if success
+                else f"[red]✗ {escape(message)}[/red]"
+            )
+            conv_log.write(status_msg)
             conv_log.write("")
         self.update_status_bar()
 
