@@ -1,5 +1,6 @@
 """Widget components for the document UI."""
 
+import os
 from typing import Any
 
 from textual.app import ComposeResult
@@ -12,10 +13,22 @@ class DocumentHeader(Static):
 
     def __init__(self, **kwargs: Any) -> None:
         super().__init__(**kwargs)
-        self.update(
-            "👀📎 clippy - 📄 Document Mode\n"
-            "Type directly, press Enter to send • Type 'y'/'n'/'stop' when prompted"
-        )
+        self.update_header()
+
+    def update_header(self) -> None:
+        """Update the header content with current working directory."""
+        cwd = os.getcwd()
+        # Shorten the path if it's too long by showing just the last 2 parts
+        if len(cwd) > 50:
+            parts = cwd.split(os.sep)
+            if len(parts) > 2:
+                cwd = os.sep.join(["..."] + parts[-2:])
+
+        self.update(f"👀📎 clippy - 📄 Document Mode\n📁 {cwd}/")
+
+    def refresh_cwd(self) -> None:
+        """Refresh the header with updated current working directory."""
+        self.update_header()
 
 
 class DocumentRibbon(Vertical):
@@ -378,8 +391,7 @@ class ApprovalDialog(Container):
             else:
                 yield Button("Yes", id="approval-yes", variant="primary")
                 yield Button("Yes (Allow All)", id="approval-allow", variant="success")
-            yield Button("No", id="approval-no", variant="default")
-            yield Button("Cancel", id="approval-stop", variant="error")
+            yield Button("No", id="approval-no", variant="error")
 
     def on_button_pressed(self, event: Button.Pressed) -> None:
         """Handle button presses for expand/collapse functionality."""
